﻿using System;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Interactivity;
using System.Windows.Media;
using System.Windows.Shapes;
using Infragistics.Controls.Charts;

namespace BarChartSelection
{
    public class SelectionBehavior : Behavior<CategorySeries>
    {
        public string SelectionPropertySuffix { get; set; }
        public bool IsStacked { get; set; }
        public string SelectionProperty { get; set; }

        private Style RectStyle;

        protected override void OnAttached()
        {
            AssociatedObject.SeriesViewer.SeriesMouseLeftButtonDown += SeriesViewer_SeriesMouseLeftButtonDown;

            if (IsStacked)
            {
                var styleType = typeof(Rectangle);
                RectStyle = new Style(styleType);
                RectStyle.Setters.Add(new EventSetter(Rectangle.LoadedEvent, new RoutedEventHandler(OnRectLoaded)));
                AssociatedObject.Resources.Add(styleType, RectStyle);
            }
        }
        protected override void OnDetaching()
        {
            if (IsStacked)
            {
                AssociatedObject.SeriesViewer.SeriesMouseLeftButtonDown -= SeriesViewer_SeriesMouseLeftButtonDown;
                AssociatedObject.Resources.Remove(RectStyle.TargetType);
                RectStyle = null;
            }
        }

        void SeriesViewer_SeriesMouseLeftButtonDown(object sender, DataChartMouseButtonEventArgs e)
        {
            if (IsStacked)
            {
                var series = e.Series as ColumnFragment;
                if (series != null)
                {
                    var selectionProperty = e.Item.GetType().GetProperty(series.ValueMemberPath + SelectionPropertySuffix);
                    if (selectionProperty != null)
                    {
                        var currentValue = (bool)selectionProperty.GetValue(e.Item);
                        selectionProperty.SetValue(e.Item, !currentValue);
                        e.Handled = true;
                    }
                }
            }
            else
            {
                var selectionProperty = e.Item.GetType().GetProperty(SelectionProperty);
                if (selectionProperty != null)
                {
                    var currentValue = (bool)selectionProperty.GetValue(e.Item);
                    selectionProperty.SetValue(e.Item, !currentValue);
                    e.Handled = true;
                }
            }
        }

        private void OnRectLoaded(object sender, RoutedEventArgs e)
        {
            var rect = sender as Rectangle;
            var dataContext = rect.DataContext as Infragistics.Controls.Charts.DataContext;
            var fragment = dataContext.Series as Infragistics.Controls.Charts.ColumnFragment;
            if (fragment != null)
            {
                var translate = rect.RenderTransform as TranslateTransform;

                var chart = AssociatedObject.SeriesViewer as XamDataChart;
                var xAxis = chart.Axes.OfType<CategoryXAxis>().First();

                // this will return the index of the item the rectangle represents
                int itemIndex = (int)Math.Round(xAxis.UnscaleValue(translate.X));
                int currentIndex = 0;
                foreach (var item in fragment.ItemsSource)
                {
                    if (currentIndex == itemIndex)
                    {
                        rect.SetBinding(
                            Rectangle.VisibilityProperty,
                            new Binding(fragment.ValueMemberPath + SelectionPropertySuffix)
                            {
                                Source = item,
                                Converter = new BooleanToVisibilityConverter()
                            });
                    }
                    currentIndex++;
                }
            }
        }
    }
}
